#!/bin/bash

HOSTAPD_CONF="/etc/hostapd/hostapd.conf"
INTERFACES="/etc/network/interfaces"
DNSMASQ_CONF="/etc/dnsmasq.conf"
DHCPCD_CONF="/etc/dhcpcd.conf"
WPA_CONF="/etc/wpa_supplicant/wpa_supplicant.conf"

BRIDGE_ID="" # empty string | value
AP_NAME=""   # empty string | Not set | value
AP_PW=""     # empty string | value

# format of sqlite3 query result:
# <key>|<value>

OLD_AP_NAME=""
OLD_AP_PW=""

while [ 1 ]
do

  ## query ap name
  sqlite3 /home/pi/.local/share/dresden-elektronik/deCONZ/zll.db "select * from config2 where key='wifiname'" > /dev/null

  if [[ $? == 0 ]]
  then
    AP_NAME=$(sqlite3 /home/pi/.local/share/dresden-elektronik/deCONZ/zll.db "select * from config2 where key='wifiname'" | cut -d'|' -f2)
  fi

  ## query ap pw
  sqlite3 /home/pi/.local/share/dresden-elektronik/deCONZ/zll.db "select * from config2 where key='wifiappw'" > /dev/null

  if [[ $? == 0 ]]
  then
    AP_PW=$(sqlite3 /home/pi/.local/share/dresden-elektronik/deCONZ/zll.db "select * from config2 where key='wifiappw'" | cut -d'|' -f2)
  fi

  ## continue configuration when both ap-name and ap-pw are set
  if [[ ($AP_NAME == "" || $AP_NAME == "Not set") || $AP_PW == "" ]]
  then
    sleep 10
    continue
  fi

  # reconfigure if parameter changed
  if [[ $AP_NAME != $OLD_AP_NAME || $AP_PW != $OLD_AP_PW ]]
  then

  echo "configuring wifi..."

  OLD_AP_NAME=$AP_NAME
  OLD_AP_PW=$AP_PW

  ## create hostapd conf
  echo "# Interface and driver
interface=wlan0
#driver=nl80211

# AP configuration
ssid=$AP_NAME
channel=11
hw_mode=g
wmm_enabled=1
country_code=DE
ieee80211d=1
ignore_broadcast_ssid=0
auth_algs=1

# AP security
wpa=2
wpa_key_mgmt=WPA-PSK
rsn_pairwise=CCMP
wpa_passphrase=$AP_PW" > $HOSTAPD_CONF

  ## Add conf to hostapd daemon
  sed -i '/DAEMON_CONF=/c\DAEMON_CONF=\"\/etc\/hostapd\/hostapd.conf\"' /etc/default/hostapd

  ## Static address for access point
  if [[ ! $(grep "interface wlan0" $DHCPCD_CONF) ]]
  then
    echo "interface wlan0
  static ip_address=192.168.8.1/24" >> $DHCPCD_CONF
  fi

  ## Add wlan0 interface
  echo "source-directory /etc/network/interfaces.d
auto wlan0
allow-hotplug wlan0
iface wlan0 inet manual" > $INTERFACES

  ## Configure dns server
  echo "interface=wlan0
no-dhcp-interface=lo,eth0
dhcp-range=192.168.8.50,192.168.8.150,24h
dhcp-option=option:dns-server,192.168.8.1" > $DNSMASQ_CONF

  echo "done!"

  ## restart network
  echo "restarting network..."
  ifdown wlan0
  wait $!
  ifup wlan0
  wait $!

  systemctl restart hostapd
  wait $!

  systemctl restart dnsmasq
  wait $!
  echo "done!"
   
fi # reconfigure if parameter changed
  
  # restart network if network problems occur
  sleep 5
  if [[ ! $(ifconfig | grep wlan0) ]]
    then
      echo "wlan0 is down - restarting network..."
      ifdown wlan0
      wait $!
      ifup wlan0
      wait $!
      echo "done!"
    fi
  
    if [[ ! $(systemctl status hostapd | grep "active (running)") ]]
    then
      echo "hostapd is inactive - restarting hostapd..."
      systemctl restart hostapd
      wait $!
      echo "done!"
	  continue
    fi

    if [[ ! $(systemctl status dnsmasq | grep "active (running)") ]]
    then
      echo "dnsmasq is inactive - restarting dnsmasq..."
      systemctl restart dnsmasq
      wait $!
      echo "done!"
    fi
  sleep 30
done

